"""Compute Folder plugin for VSS CLI (vss-cli)."""
import logging

import click

from vss_cli import const, rel_opts as so
import vss_cli.autocompletion as autocompletion
from vss_cli.cli import pass_context
from vss_cli.config import Configuration
from vss_cli.exceptions import VssCliError
from vss_cli.helper import format_output
from vss_cli.plugins.compute import cli

_LOGGING = logging.getLogger(__name__)


@cli.group('folder', short_help='Manage logical folders')
@pass_context
def compute_folder(ctx):
    """Logical Folders command.

    Folders are containers for storing and organizing
    inventory objects, in this case virtual machines.
    """


@compute_folder.command('ls', short_help='list folders')
@so.filter_opt
@so.sort_opt
@so.all_opt
@so.page_opt
@pass_context
def compute_folder_ls(ctx: Configuration, filter_by, show_all, sort, page):
    """List logical folders based on.

    Filter list in the following format <field_name>=<operator>,<value>
    where operator is eq, ne, lt, le, gt, ge, like, in.

    For example: name like,%Project%

    vss-cli compute folder ls -f name=like,%Project%

    Sort list in the following format <field_name>=<asc|desc>. For example:

    vss-cli compute folder ls -s path=desc
    """
    params = dict(expand=1, sort='path,asc')
    if all(filter_by):
        params['filter'] = ';'.join(filter_by)
    if all(sort):
        params['sort'] = ';'.join(sort)
    # get objects
    with ctx.spinner(disable=ctx.debug):
        obj = ctx.get_folders(show_all=show_all, **params)
    # set columns
    columns = ctx.columns or const.COLUMNS_FOLDER_MIN
    # format
    output = format_output(ctx, obj, columns=columns)
    # page results
    if page:
        click.echo_via_pager(output)
    else:
        ctx.echo(output)


@compute_folder.group('set', short_help='update folder')
@click.argument(
    'moref_or_name', type=click.STRING, autocompletion=autocompletion.domains
)
@so.wait_opt
@pass_context
def compute_folder_set(ctx, moref_or_name: str, wait: bool):
    """Update given folder attribute."""
    _folder = ctx.get_folder_by_name_or_moref_path(moref_or_name)
    ctx.moref = _folder[0]['moref']
    ctx.wait = wait


@compute_folder_set.command('parent', short_help='move folder')
@click.argument('parent-name-or-moref', type=click.STRING, required=True)
@pass_context
def compute_folder_set_parent(ctx: Configuration, parent_name_or_moref):
    """Move folder to given moref.

    Use to obtain parent folder:

    vss-cli compute folder ls
    """
    _LOGGING.debug(f'Attempting to move {ctx.moref} to {parent_name_or_moref}')
    # exist parent
    _folder = ctx.get_folder_by_name_or_moref_path(parent_name_or_moref)
    parent_moref = _folder[0]['moref']
    # create payload
    payload = dict(moref=ctx.moref, new_moref=parent_moref)
    obj = ctx.move_folder(**payload)
    # format output
    columns = ctx.columns or const.COLUMNS_REQUEST_SUBMITTED
    ctx.echo(format_output(ctx, [obj], columns=columns, single=True))
    # wait for request
    if ctx.wait:
        ctx.wait_for_request_to(obj)


@compute_folder_set.command('name', short_help='rename folder')
@click.argument('name', type=click.STRING, required=True)
@pass_context
def compute_folder_set_name(ctx: Configuration, name):
    """Rename folder to given name.

    Use to obtain parent folder:

    vss-cli compute folder ls
    """
    _LOGGING.debug(f'Attempting to rename {ctx.moref} to {name}')
    # exist folder
    payload = dict(moref=ctx.moref, name=name)
    obj = ctx.rename_folder(**payload)
    # format output
    columns = ctx.columns or const.COLUMNS_REQUEST_SUBMITTED
    ctx.echo(format_output(ctx, [obj], columns=columns, single=True))
    # wait for request
    if ctx.wait:
        ctx.wait_for_request_to(obj)


@compute_folder.command('rm', short_help='remove folder')
@click.argument(
    'moref',
    type=click.STRING,
    required=True,
    nargs=-1,
    autocompletion=autocompletion.folders,
)
@so.max_del_opt
@so.wait_opt
@pass_context
def compute_folder_rm(
    ctx: Configuration, moref: str, max_del: int, wait: bool
):
    """Delete a logical folder.

    Note. Folder must be empty.
    """
    _LOGGING.debug(f'Attempting to remove {moref}')
    if len(moref) > max_del:
        raise click.BadArgumentUsage(
            'Increase max instance removal with --max-del/-m option'
        )
    objs = list()
    for folder in moref:
        skip = False
        _f = ctx.get_folder_by_name_or_moref_path(folder, silent=True)
        if not _f:
            _LOGGING.warning(f'Folder {folder} could not be found. Skipping.')
            skip = True

        if not skip and _f:
            mo_id = _f[0]['moref']
            objs.append(ctx.delete_folder(moref=mo_id))
    # print
    columns = ctx.columns or const.COLUMNS_REQUEST_SUBMITTED
    ctx.echo(format_output(ctx, objs, columns=columns, single=len(objs) == 1))
    if wait:
        if len(objs) > 1:
            ctx.wait_for_requests_to(objs, in_multiple=True)
        else:
            ctx.wait_for_request_to(objs[0])


@compute_folder.command('mk', short_help='create folder')
@click.argument(
    'name', type=click.STRING, required=True, nargs=-1,
)
@click.option(
    '-p',
    '--parent',
    type=click.STRING,
    required=True,
    help='Parent folder name, path or moref',
    autocompletion=autocompletion.folders,
)
@so.wait_opt
@pass_context
def compute_folder_mk(ctx: Configuration, parent, name: list, wait: bool):
    """Create a logical folder under a given name, path or moref of parent."""
    _LOGGING.debug(f'Attempting to create {name} under {parent}')
    # exist folder
    _parent = ctx.get_folder_by_name_or_moref_path(parent)
    objs = list()
    parent = _parent[0]['moref']
    for child in name:
        objs.append(ctx.create_folder(moref=parent, name=child))
    # print
    columns = ctx.columns or const.COLUMNS_REQUEST_SUBMITTED
    ctx.echo(format_output(ctx, objs, columns=columns, single=len(objs) == 1))
    if wait:
        if len(objs) > 1:
            ctx.wait_for_requests_to(objs, in_multiple=True)
        else:
            ctx.wait_for_request_to(objs[0])


@compute_folder.group(
    'get', help='Given folder info.', invoke_without_command=True
)
@click.argument(
    'moref_or_name',
    type=click.STRING,
    required=True,
    autocompletion=autocompletion.folders,
)
@pass_context
def compute_folder_get(ctx: Configuration, moref_or_name):
    """Get logical folder information."""
    _folder = ctx.get_folder_by_name_or_moref_path(moref_or_name)
    ctx.moref = _folder[0]['moref']
    if click.get_current_context().invoked_subcommand is None:
        obj = ctx.get_folder(ctx.moref)
        obj['moref'] = ctx.moref
        # set columns
        columns = ctx.columns or const.COLUMNS_FOLDER
        # format
        ctx.echo(format_output(ctx, [obj], columns=columns, single=True))


@compute_folder_get.command('vms', short_help='list virtual machines.')
@click.option(
    '-p', '--page', is_flag=True, help='page results in a less-like format'
)
@pass_context
def compute_folder_get_vms(ctx: Configuration, page):
    """List logical folder children virtual machines."""
    obj = ctx.get_vms_by_folder(ctx.moref)
    # format output
    columns = ctx.columns or const.COLUMNS_VM_MIN
    output = format_output(ctx, obj, columns=columns)
    # page
    if page:
        click.echo_via_pager(output)
    else:
        ctx.echo(output)


@compute_folder_get.command('children', short_help='list children folders.')
@click.option(
    '-p', '--page', is_flag=True, help='page results in a less-like format'
)
@pass_context
def compute_folder_get_children(ctx: Configuration, page):
    """List logical folder immediate children folders."""
    obj = ctx.get_folder_children(ctx.moref)
    # format output
    columns = ctx.columns or const.COLUMNS_FOLDER_MIN
    output = format_output(ctx, obj, columns=columns)
    # page
    if page:
        click.echo_via_pager(output)
    else:
        ctx.echo(output)


@compute_folder_get.command('perm', short_help='list permissions.')
@click.option(
    '-p', '--page', is_flag=True, help='page results in a less-like format'
)
@pass_context
def compute_folder_get_perms(ctx: Configuration, page):
    """Get logical folder group or user permissions."""
    obj = ctx.get_folder_permission(ctx.moref)
    if not obj:
        raise VssCliError(
            f'Either folder {ctx.moref} does not exist, '
            f'or you do not have permission to access.'
        )
    columns = ctx.columns or const.COLUMNS_PERMISSION
    output = format_output(ctx, obj, columns=columns)
    # page
    if page:
        click.echo_via_pager(output)
    else:
        ctx.echo(output)



"""Misc plugin for VSS CLI (vss-cli)."""
import logging

import click

from vss_cli.cli import pass_context
from vss_cli.config import Configuration

_LOGGING = logging.getLogger(__name__)


@click.group('misc', short_help='Miscellaneous utilities.')
@pass_context
def cli(ctx):
    """Execute Miscellaneous utilities."""


@cli.command('hash-string', short_help='Hashes string using SHA-512')
@click.argument('string_to_hash', type=click.STRING, required=False)
@pass_context
def misc_hash_string(ctx: Configuration, string_to_hash):
    """Hash string using SHA-512.

    If string_to_hash argument not provided, input will be prompted.
    Useful to create hashed passwd entries.
    """
    from pyvss.helper import hash_string

    string_to_hash = string_to_hash or click.prompt(
        'String', show_default=False, hide_input=True, confirmation_prompt=True
    )
    ctx.echo(hash_string(string_to_hash))


@cli.command('gz-b64e', short_help='Compresses and encodes a given string')
@click.argument('string_gz_encode', type=click.STRING, required=True)
@pass_context
def misc_encode_gz(ctx: Configuration, string_gz_encode):
    """Compress (gz) and encode in base64 string."""
    from pyvss.helper import compress_encode_string

    ctx.echo(compress_encode_string(string_gz_encode))


@cli.command('b64d-gz', short_help='Decompress and decodes a given string')
@click.argument('string_gz_encoded', type=click.STRING, required=True)
@pass_context
def misc_decodes_ugz(ctx: Configuration, string_gz_encoded):
    """Decompress (gz) and decode string."""
    from pyvss.helper import decode_uncompress_string

    ctx.echo(decode_uncompress_string(string_gz_encoded))



#!/usr/bin/env python3
"""Setup script for ITS Private Cloud CLI."""
import codecs
from datetime import datetime as dt
import io
import os
import re
from typing import List

from setuptools import find_packages, setup

# shared consts using approach suggested at
# https://stackoverflow.com/questions/17583443/what-is-the-correct-way-to-share-package-version-with-setup-py-and-the-package


def read(*parts):
    """Read file from current directory."""
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, *parts), 'r') as infofile:
        return infofile.read()


def find_version(*file_paths):
    """Locate version info to share between const.py and setup.py."""
    version_file = read(*file_paths)  # type: ignore
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M
    )
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def load_requirements(requires_file: str = 'requirements.txt') -> List[str]:
    """Load requirements from file"""
    with io.open(requires_file, encoding='utf-8') as f:
        return f.read().splitlines()


__VERSION__ = find_version("vss_cli", "const.py")  # type: ignore

REQUIRED_PYTHON_VER = (3, 6, 4)
REQUIRES = load_requirements()

PACKAGES = find_packages(exclude=['tests', 'tests.*'])
PACKAGE_DATA = {'vss_cli': ['data/*.yaml']}

PROJECT_NAME = 'ITS Private Cloud CLI'
PROJECT_PACKAGE_NAME = 'vss-cli'
PROJECT_LICENSE = 'MIT'
PROJECT_AUTHOR = 'University of Toronto'
PROJECT_COPYRIGHT = f' 2019-{dt.now().year}, {PROJECT_AUTHOR}'
PROJECT_URL = 'https://gitlab-ee.eis.utoronto.ca/vss/vss-cli'
PROJECT_DOCS = 'https://eis.utoronto.ca/~vss/vss-cli'
PROJECT_EMAIL = 'vss-apps@eis.utoronto.ca'
MAINTAINER_EMAIL = 'vss-py@eis.utoronto.ca'

PROJECT_GITLAB_GROUP = 'vss'
PROJECT_GITLAB_REPOSITORY = 'vss-cli'

PYPI_URL = f'https://pypi.python.org/pypi/{PROJECT_PACKAGE_NAME}'
GITLAB_PATH = f'{PROJECT_GITLAB_GROUP}/{PROJECT_GITLAB_REPOSITORY}'
GITLAB_URL = f'https://gitlab-ee.eis.utoronto.ca/{GITLAB_PATH}'

DOWNLOAD_URL = f'{GITLAB_URL}/archive/{__VERSION__}.zip'
PROJECT_URLS = {
    'Bug Reports': f'{GITLAB_URL}/issues',
    'Documentation': f'{PROJECT_DOCS}/',
    'Source': f'{PROJECT_URL}',
}
STOR_REQUIRE = ['webdavclient3==0.12']
TESTS_REQUIRE = [
    'flake8==3.7.7',
    'nose==1.3.7',
    'coverage==4.5.3',
    'pytz==2018.9',
    'wheel==0.33.1',  # Otherwise setup.py bdist_wheel does not work
    *STOR_REQUIRE,
]
DEV_REQUIRE = [
    *TESTS_REQUIRE,
    *STOR_REQUIRE,
    'sphinx-rtd-theme==0.4.3',
    'Sphinx==1.8.5',
]

# Allow you to run
# pip install .[test]
# pip install .[dev]
# to get test dependencies included
EXTRAS_REQUIRE = {
    'test': TESTS_REQUIRE,
    'dev': DEV_REQUIRE,
    'stor': STOR_REQUIRE,
}

MIN_PY_VERSION = '.'.join(map(str, REQUIRED_PYTHON_VER))

setup(
    name=PROJECT_PACKAGE_NAME,
    version=__VERSION__,
    url=PROJECT_URL,
    download_url=DOWNLOAD_URL,
    project_urls=PROJECT_URLS,
    author=PROJECT_AUTHOR,
    author_email=PROJECT_EMAIL,
    maintainer_email=MAINTAINER_EMAIL,
    packages=PACKAGES,
    package_data=PACKAGE_DATA,
    license=PROJECT_LICENSE,
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIRES,
    tests_require=TESTS_REQUIRE,
    extras_require=EXTRAS_REQUIRE,
    python_requires=f'>={MIN_PY_VERSION}',
    entry_points={
        'console_scripts': [
            'vss-cli = vss_cli.cli:run',
            'vss = vss_cli.cli:run',
        ]
    },
)



"""Compute Domain plugin for VSS CLI (vss-cli)."""
import logging

import click

from vss_cli import const, rel_opts as so
import vss_cli.autocompletion as autocompletion
from vss_cli.cli import pass_context
from vss_cli.config import Configuration
from vss_cli.exceptions import VssCliError
from vss_cli.helper import format_output
from vss_cli.plugins.compute import cli

_LOGGING = logging.getLogger(__name__)


@cli.group('domain', short_help='List compute domains.')
@pass_context
def cli(ctx: Configuration):
    """Domain command.

    A fault domain consists of one or more ESXI hosts and
    Datastore Clusters grouped together according to their
    physical location in the datacenter.
    """


@cli.command('ls', short_help='list fault domains')
@so.filter_opt
@so.all_opt
@so.page_opt
@so.sort_opt
@so.count_opt
@pass_context
def domain_ls(ctx: Configuration, filter_by, show_all, sort, page, count):
    """List available fault domains."""
    columns = ctx.columns or const.COLUMNS_MOREF
    params = dict(expand=1, sort='name,asc')
    if all(filter_by):
        params['filter'] = ';'.join(filter_by)
    if all(sort):
        params['sort'] = ';'.join(sort)
    # query
    with ctx.spinner(disable=ctx.debug):
        objs = ctx.get_domains(show_all=show_all, per_page=count, **params)
    # format output
    output = format_output(ctx, objs, columns=columns)
    # page results
    if page:
        click.echo_via_pager(output)
    else:
        ctx.echo(output)


@cli.group('get', help='Given domain info.', invoke_without_command=True)
@click.argument(
    'name_or_moref',
    type=click.STRING,
    required=True,
    autocompletion=autocompletion.domains,
)
@pass_context
def domain_get(ctx: Configuration, name_or_moref):
    """Get fault domain information."""
    _domain = ctx.get_domain_by_name_or_moref(name_or_moref)
    ctx.moref = _domain[0]['moref']
    if click.get_current_context().invoked_subcommand is None:
        columns = ctx.columns or const.COLUMNS_MOREF
        with ctx.spinner(disable=ctx.debug):
            obj = ctx.get_domain(ctx.moref)
        ctx.echo(format_output(ctx, [obj], columns=columns, single=True))


@domain_get.command('vms', help='Given domain vms.')
@click.option(
    '-p', '--page', is_flag=True, help='page results in a less-like format'
)
@pass_context
def domain_get_vms(ctx: Configuration, page):
    """Get VMs in given fault domain."""
    with ctx.spinner(disable=ctx.debug):
        obj = ctx.get_vms_by_domain(ctx.moref)
    if not obj:
        raise VssCliError(
            f'Either domain {ctx.moref} does not exist, '
            f'or you do not have permission to access.'
        )
    output = format_output(ctx, obj, columns=const.COLUMNS_VM)
    # page results
    if page:
        click.echo_via_pager(output)
    else:
        ctx.echo(output)



"""Message Management plugin for VSS CLI (vss-cli)."""
import click

from vss_cli import const, rel_opts as so
import vss_cli.autocompletion as autocompletion
from vss_cli.cli import pass_context
from vss_cli.config import Configuration
from vss_cli.helper import format_output


@click.group('message', short_help='Manage VSS Messages.')
@pass_context
def cli(ctx: Configuration):
    """Manage VSS Messages."""
    with ctx.spinner(disable=ctx.debug):
        ctx.load_config()


@cli.command('ls', short_help='list user message')
@so.filter_opt
@so.sort_opt
@so.all_opt
@so.count_opt
@so.page_opt
@pass_context
def message_ls(ctx: Configuration, filter_by, page, sort, show_all, count):
    """List messages.

    Filter list in the following format <field_name>=<operator>,<value>
    where operator is eq, ne, lt, le, gt, ge, like, in.
    For example: kind eq,Notice

    vss-cli message ls -f kind=eq,Notice

    Sort list in the following format <field_name>=<asc|desc>. For example:

    vss-cli message ls -s created_on=desc
    """
    columns = ctx.columns or const.COLUMNS_MESSAGE_MIN
    params = dict(expand=1, sort='status,asc')
    if all(filter_by):
        params['filter'] = ';'.join(filter_by)
    if all(sort):
        params['sort'] = ';'.join(sort)
    # make request
    with ctx.spinner(disable=ctx.debug):
        _requests = ctx.get_user_messages(
            show_all=show_all, per_page=count, **params
        )
    # format output
    output = format_output(ctx, _requests, columns=columns)
    # page results
    if page:
        click.echo_via_pager(output)
    else:
        click.echo(output)


@cli.command('get', help='Display user message info.')
@click.argument(
    'message_id',
    type=click.INT,
    required=True,
    autocompletion=autocompletion.account_messages,
)
@pass_context
def message_get(ctx: Configuration, message_id):
    """Get message info."""
    with ctx.spinner(disable=ctx.debug):
        obj = ctx.get_user_message(message_id)
    columns = ctx.columns or const.COLUMNS_MESSAGE
    click.echo(format_output(ctx, [obj], columns=columns, single=True))


@cli.group('set', short_help='Set given user message attribute.')
@pass_context
def message_set(ctx):
    """Update message info."""


@message_set.command('ack', short_help='Acknowledge user message')
@click.argument(
    'message_id',
    type=click.INT,
    required=True,
    nargs=-1,
    autocompletion=autocompletion.account_messages,
)
@click.option('-s', '--summary', is_flag=True, help='Print request summary')
@pass_context
def message_set_ack(ctx, message_id, summary):
    """Acknowledge message."""
    result = []
    with click.progressbar(message_id) as ids:
        for i in ids:
            result.append(ctx.ack_user_message(i))
    if summary:
        for res in result:
            click.echo(
                format_output(
                    ctx, [res], columns=[('STATUS', 'status')], single=True
                )
            )
